/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.pmd;

import ch.hortis.sonar.model.RuleFailureLevel;
import org.sonar.plugins.api.maven.*;
import org.sonar.plugins.api.rules.RulesManager;
import org.w3c.dom.Element;

public class PmdFailureCollector extends AbstractFailureCollector{
  protected PmdFailureCollector(JavaMeasuresRecorder measuresRecorder, RulesManager rulesManager) {
    super(measuresRecorder, rulesManager);
  }

  @Override
  protected String xpathForFiles() {
    return "/pmd/file";
  }

  @Override
  protected String elementNameForFailures() {
    return "violation";
  }

  @Override
  protected String javaFilePath(Element element) {
    return element.getAttribute("name");
  }

  @Override
  protected String ruleKey(Element failure) {
    return failure.getAttribute("rule");
  }

  @Override
  protected String keyForPlugin() {
    return PmdPlugin.KEY;
  }

  @Override
  protected RuleFailureLevel getLevel(Element failure) {
    RuleFailureLevel level = RuleFailureLevel.INFO;
    int priority = Integer.parseInt(failure.getAttribute("priority"));
    if (priority <= 2) {
      level = RuleFailureLevel.ERROR;
    }
    else if (priority == 3 || priority == 4) {
      level = RuleFailureLevel.WARNING;
    }
    return level;
  }

  @Override
  protected String lineNumberFor(Element failure) {
    return failure.getAttribute("beginline");
  }

  @Override
  protected String messageFor(Element failure) {
    return failure.getFirstChild().getNodeValue();
  }
}
