/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.pmd;

import org.apache.commons.io.FileUtils;
import org.sonar.plugins.api.Languages;
import org.sonar.plugins.api.maven.*;
import org.sonar.plugins.api.maven.model.*;
import org.sonar.plugins.api.rules.RulesManager;

import java.io.*;

public class PmdMavenPluginHandler extends AbstractMavenPluginHandler {
  public static final String GROUP_ID = MavenUtils.APACHE_MOJO_GROUP_ID;
  public static final String ARTIFACT_ID = "maven-pmd-plugin";
  public static final String VERSION = "2.4";

  private RulesManager rulesManager;
  private PmdRulesRepository pmdRulesRepository;

  public PmdMavenPluginHandler(RulesManager rulesManager, PmdRulesRepository pmdRulesRepository) {
    this.rulesManager = rulesManager;
    this.pmdRulesRepository = pmdRulesRepository;
  }

  public String getGroupId() {
    return GROUP_ID;
  }

  public String getArtifactId() {
    return ARTIFACT_ID;
  }

  public String getVersion() {
    return VERSION;
  }

  public boolean isFixedVersion() {
    return true;
  }

  public String[] getGoals() {
    return new String[]{"pmd"};
  }

  public boolean shouldStopOnFailure() {
    return true;
  }

  @Override
  public void configurePlugin(MavenPom pom, MavenPlugin plugin) {
    plugin.unsetConfigParameter( "outputDirectory");
    plugin.unsetConfigParameter( "targetDirectory");
    plugin.setConfigParameter( "format", "xml");
    plugin.setConfigParameter( "linkXRef", "false");

    String javaVersion = pom.getJavaVersion();
    if (javaVersion != null) {
      if ("1.1".equals(javaVersion) || "1.2".equals(javaVersion)) {
        plugin.setConfigParameter( "targetJdk", "1.3");
      } else {
        plugin.setConfigParameter( "targetJdk", javaVersion);
      }
    }
    plugin.unsetConfigParameter( "rulesets");

    MavenPluginConfiguration config = plugin.getConfiguration();
    config.setChildValue("rulesets/ruleset", pom.getBuildDirectory() + "/sonar/pmd.xml");

    plugin.copyDependenciesFrom(pom);
    try {
      File xmlFile = saveConfigXml(pom);
      plugin.setConfigParameter( "configLocation", xmlFile.getCanonicalPath());
    }
    catch (IOException e) {
      throw new RuntimeException("fail to save the pmd XML configuration", e);
    }

    addRuleExtensionsDependency(plugin);
  }

  private File saveConfigXml(MavenPom pom) throws IOException {
    File pmd = new File(pom.getSonarWorkingDirectory(), "pmd.xml");
    String xml = pmdRulesRepository.exportConfiguration(rulesManager.getActiveProfile(Languages.JAVA));
    FileUtils.writeStringToFile(pmd, xml);
    return pmd;
  }
}
