/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.pmd.cpd;

import ch.hortis.sonar.model.Metric;
import ch.hortis.sonar.service.MeasureKey;
import org.sonar.commons.measures.Measure;
import org.sonar.plugins.api.Language;
import org.sonar.plugins.api.Languages;
import org.sonar.plugins.api.jobs.AbstractJob;
import org.sonar.plugins.api.jobs.Resource;
import org.sonar.plugins.api.metrics.CoreMetrics;

import java.util.Arrays;
import java.util.List;

public class DuplicatedLinesRatioJob extends AbstractJob {

  protected Language shouldExecuteOnLanguage() {
    return Languages.JAVA;
  }

  public boolean shouldExecuteOnResource(Resource node) {
    return !node.isClass();
  }

  @Override
  public List<Metric> dependsOnMetrics() {
    return Arrays.asList(CoreMetrics.DUPLICATED_LINES, CoreMetrics.NLOC, CoreMetrics.COMMENT_LINES);
  }

  @Override
  public List<Metric> generatesMetrics() {
    return Arrays.asList(CoreMetrics.DUPLICATED_LINES_RATIO);
  }

  public void execute(Resource node) {
    Measure nbDuplicatedLines = node.getMeasure(new MeasureKey(CoreMetrics.DUPLICATED_LINES));
    Measure nbLines = node.getMeasure(new MeasureKey(CoreMetrics.NLOC));
    Measure nbComments = node.getMeasure(new MeasureKey(CoreMetrics.COMMENT_LINES));

    if (nbDuplicatedLines != null && nbLines != null) {
      Double divisor = nbLines.getValue();
      if (nbComments != null) {
        divisor += nbComments.getValue();
      }
      if (divisor > 0.0) {
        node.addMeasure(CoreMetrics.DUPLICATED_LINES_RATIO, calculate(nbDuplicatedLines.getValue(), divisor));
      }
    }
  }

  protected Double calculate(Double dividend, Double divisor) {
    Double result = 100.0 * dividend / divisor;
    if (result < 100.0) {
      return result;
    } else {
      return 100.0;
    }
  }

}